@@ -1,13 +1,13 @@
 r"""Parser for python source files."""
+
 # Created Sun Nov 27 14:03:07 2016
 # Author: Óscar Nájera
 
-
-import codecs
 import ast
-from io import BytesIO
 import re
 import tokenize
+from collections import namedtuple
+from io import BytesIO
 from textwrap import dedent
 
 from sphinx.errors import ExtensionError
@@ -33,9 +33,8 @@ Example script with invalid Python syntax
 #
 #     b = 2
 FLAG_START = r"^[\ \t]*#\s*"
-INFILE_CONFIG_PATTERN = re.compile(
-    FLAG_START + r"sphinx_gallery_([A-Za-z0-9_]+)(\s*=\s*(.+))?[\ \t]*\n?", re.MULTILINE
-)
+FLAG_BODY = r"sphinx_gallery_([A-Za-z0-9_]+)(\s*=\s*(.+))?[\ \t]*\n?"
+INFILE_CONFIG_PATTERN = re.compile(FLAG_START + FLAG_BODY, re.MULTILINE)
 
 START_IGNORE_FLAG = FLAG_START + "sphinx_gallery_start_ignore"
 END_IGNORE_FLAG = FLAG_START + "sphinx_gallery_end_ignore"
@@ -57,10 +56,9 @@ def parse_source_file(filename):
     node : AST node
     content : utf-8 encoded string
     """
-    with codecs.open(filename, "r", "utf-8") as fid:
+    # builtin open automatically converts \r\n to \n
+    with open(filename, "r", encoding="utf-8") as fid:
         content = fid.read()
-    # change from Windows format to UNIX for uniformity
-    content = content.replace("\r\n", "\n")
 
     try:
         node = ast.parse(content)
@@ -93,13 +91,14 @@ def _get_docstring_and_rest(filename):
 
     if not isinstance(node, ast.Module):
         raise ExtensionError(
-            "This function only supports modules. "
-            "You provided {}".format(node.__class__.__name__)
+            "This function only supports modules. " "You provided {}".format(
+                node.__class__.__name__
+            )
         )
     if not (
         node.body
         and isinstance(node.body[0], ast.Expr)
-        and isinstance(node.body[0].value, ast.Str)
+        and isinstance(node.body[0].value, ast.Constant)
     ):
         raise ExtensionError(
             'Could not find docstring in file "{}". '
@@ -111,7 +110,7 @@ def _get_docstring_and_rest(filename):
     docstring = ast.get_docstring(node)
     assert docstring is not None  # should be guaranteed above
     # This is just for backward compat
-    if len(node.body[0].value.s) and node.body[0].value.s[0] == "\n":
+    if node.body[0].value.value[:1] == "\n":
         # just for strict backward compat here
         docstring = "\n" + docstring
     ts = tokenize.tokenize(BytesIO(content.encode()).readline)
@@ -149,6 +148,12 @@ def extract_file_config(content):
     return file_conf
 
 
+Block = namedtuple("Block", ["type", "content", "lineno"])
+# type: "text" or "code"
+# content (str): the block lines as str
+# lineno (int): the line number where the block starts
+
+
 def split_code_and_text_blocks(source_file, return_node=False):
     """Return list with source file separated into code and text blocks.
 
@@ -172,7 +177,7 @@ def split_code_and_text_blocks(source_file, return_node=False):
         The parsed ast node.
     """
     docstring, rest_of_content, lineno, node = _get_docstring_and_rest(source_file)
-    blocks = [("text", docstring, 1)]
+    blocks = [Block("text", docstring, 1)]
 
     file_conf = extract_file_config(rest_of_content)
 
@@ -186,21 +191,21 @@ def split_code_and_text_blocks(source_file, return_node=False):
     for match in re.finditer(pattern, rest_of_content):
         code_block_content = rest_of_content[pos_so_far : match.start()]
         if code_block_content.strip():
-            blocks.append(("code", code_block_content, lineno))
+            blocks.append(Block("code", code_block_content, lineno))
         lineno += code_block_content.count("\n")
 
         lineno += 1  # Ignored header line of hashes.
         text_content = match.group("text_content")
         text_block_content = dedent(re.sub(sub_pat, "", text_content)).lstrip()
         if text_block_content.strip():
-            blocks.append(("text", text_block_content, lineno))
+            blocks.append(Block("text", text_block_content, lineno))
         lineno += text_content.count("\n")
 
         pos_so_far = match.end()
 
     remaining_content = rest_of_content[pos_so_far:]
     if remaining_content.strip():
-        blocks.append(("code", remaining_content, lineno))
+        blocks.append(Block("code", remaining_content, lineno))
 
     out = (file_conf, blocks)
     if return_node:
@@ -212,7 +217,7 @@ def remove_ignore_blocks(code_block):
     """
     Return the content of *code_block* with ignored areas removed.
 
-    An ignore block starts with # sphinx_gallery_begin_ignore, and ends with
+    An ignore block starts with # sphinx_gallery_start_ignore, and ends with
     # sphinx_gallery_end_ignore. These lines and anything in between them will
     be removed, but surrounding empty lines are preserved.
 
@@ -221,8 +226,8 @@ def remove_ignore_blocks(code_block):
     code_block : str
         A code segment.
     """
-    num_start_flags = len(re.findall(START_IGNORE_FLAG, code_block))
-    num_end_flags = len(re.findall(END_IGNORE_FLAG, code_block))
+    num_start_flags = len(re.findall(START_IGNORE_FLAG, code_block, re.MULTILINE))
+    num_end_flags = len(re.findall(END_IGNORE_FLAG, code_block, re.MULTILINE))
 
     if num_start_flags != num_end_flags:
         raise ExtensionError(
