@@ -2,27 +2,37 @@
 # License: 3-clause BSD
 r"""Test Sphinx-Gallery gallery generation."""
 
-import codecs
+import json
 import os
 import re
 from pathlib import Path
-import json
 
 import pytest
-
 from sphinx.config import is_serializable
 from sphinx.errors import ConfigError, ExtensionError, SphinxWarning
+
 from sphinx_gallery.gen_gallery import (
-    check_duplicate_filenames,
-    check_spaces_in_filenames,
-    collect_gallery_files,
-    write_computation_times,
     _fill_gallery_conf_defaults,
-    write_api_entry_usage,
     fill_gallery_conf_defaults,
+    write_api_entry_usage,
+    write_computation_times,
 )
 from sphinx_gallery.interactive_example import create_jupyterlite_contents
-from sphinx_gallery.utils import _escape_ansi
+from sphinx_gallery.utils import (
+    _collect_gallery_files,
+    _escape_ansi,
+    check_duplicate_filenames,
+    check_spaces_in_filenames,
+)
+
+MINIMAL_HEADER = """
+'''
+Title
+-----
+Description.
+'''
+
+"""
 
 
 def test_bad_config():
@@ -70,7 +80,7 @@ def test_serializable(sphinx_app_wrapper):
     assert not bad, f"Non-serializable values found:\n{bad}"
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -90,6 +100,20 @@ def test_no_warning_simple_config(sphinx_app_wrapper):
     assert build_warn == ""
 
 
+@pytest.mark.add_conf(
+    content="""
+sphinx_gallery_conf = {
+    'examples_dirs': ['src', 'excess'],
+    'gallery_dirs': 'ex',
+}"""
+)
+def test_unequal_examples_gallery_dirs(sphinx_app_wrapper):
+    """Check warning when 'examples_dirs' and 'gallery_dirs' unequal lengths."""
+    sphinx_app = sphinx_app_wrapper.create_sphinx_app()
+    build_warn = sphinx_app._warning.getvalue()
+    assert "'examples_dirs' and 'gallery_dirs' are of different lengths" in build_warn
+
+
 # This changed from passing the ValueError directly to
 # raising "sphinx.errors.ConfigError" with "threw an exception"
 @pytest.mark.parametrize(
@@ -99,7 +123,7 @@ def test_no_warning_simple_config(sphinx_app_wrapper):
             ConfigError,
             "Unknown string option for reset_modules",
             id="Resetter unknown",
-            marks=pytest.mark.conf_file(
+            marks=pytest.mark.add_conf(
                 content="sphinx_gallery_conf={'reset_modules': ('f',)}"
             ),
         ),
@@ -107,7 +131,7 @@ def test_no_warning_simple_config(sphinx_app_wrapper):
             ConfigError,
             "reset_modules.* must be callable",
             id="Resetter not callable",
-            marks=pytest.mark.conf_file(
+            marks=pytest.mark.add_conf(
                 content="sphinx_gallery_conf={'reset_modules': (1.,),}"
             ),
         ),
@@ -123,9 +147,9 @@ def test_bad_reset(sphinx_app_wrapper, err_class, err_match):
     [
         pytest.param(
             ConfigError,
-            "reset_modules_order must be a str",
+            "'reset_modules_order' config allowed types",
             id="Resetter unknown",
-            marks=pytest.mark.conf_file(
+            marks=pytest.mark.add_conf(
                 content=("sphinx_gallery_conf=" "{'reset_modules_order': 1,}")
             ),
         ),
@@ -133,7 +157,7 @@ def test_bad_reset(sphinx_app_wrapper, err_class, err_match):
             ConfigError,
             "reset_modules_order must be in",
             id="reset_modules_order not valid",
-            marks=pytest.mark.conf_file(
+            marks=pytest.mark.add_conf(
                 content=("sphinx_gallery_conf=" "{'reset_modules_order': 'invalid',}")
             ),
         ),
@@ -151,15 +175,13 @@ def test_bad_reset_modules_order(sphinx_app_wrapper, err_class, err_match):
             ConfigError,
             "Unknown css",
             id="CSS str error",
-            marks=pytest.mark.conf_file(
-                content="sphinx_gallery_conf={'css': ('foo',)}"
-            ),
+            marks=pytest.mark.add_conf(content="sphinx_gallery_conf={'css': ('foo',)}"),
         ),
         pytest.param(
             ConfigError,
-            "must be list or tuple",
+            "config allowed types:",
             id="CSS type error",
-            marks=pytest.mark.conf_file(content="sphinx_gallery_conf={'css': 1.}"),
+            marks=pytest.mark.add_conf(content="sphinx_gallery_conf={'css': 1.}"),
         ),
     ],
 )
@@ -172,14 +194,14 @@ def test_bad_css(sphinx_app_wrapper, err_class, err_match):
 def test_bad_api():
     """Test that we raise an error for bad API usage arguments."""
     sphinx_gallery_conf = dict(api_usage_ignore=("foo",))
-    with pytest.raises(ConfigError, match=".*must be str.*"):
+    with pytest.raises(ConfigError, match="'api_usage_ignore' config allowed"):
         _fill_gallery_conf_defaults(sphinx_gallery_conf)
     sphinx_gallery_conf = dict(show_api_usage="foo")
     with pytest.raises(ConfigError, match='.*must be True, False or "unused".*'):
         _fill_gallery_conf_defaults(sphinx_gallery_conf)
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'backreferences_dir': os.path.join('gen_modules', 'backreferences'),
@@ -227,7 +249,7 @@ def test_spaces_in_files_warn(sphinx_app_wrapper):
 
     files = ["./a/file1.py", "./a/file2.py", "./a/file 3.py"]
     msg = (
-        "Example file name(s) with space(s) found. Having space(s) in "
+        "Example file name(s) with spaces found. Having spaces in "
         "file names will break some links. "
         "List of files: {}"
     )
@@ -249,10 +271,10 @@ def _check_order(sphinx_app, key):
 
     Used to test that these keys (in index.rst) appear in a specific order.
     """
-    index_fname = os.path.join(sphinx_app.outdir, "..", "ex", "index.rst")
+    index_fname = Path(sphinx_app.outdir, "..", "ex", "index.rst")
     order = list()
     regex = f".*:{key}=(.):.*"
-    with codecs.open(index_fname, "r", "utf-8") as fid:
+    with open(index_fname, "r", encoding="utf-8") as fid:
         for line in fid:
             if "sphx-glr-thumbcontainer" in line:
                 order.append(int(re.match(regex, line).group(1)))
@@ -260,7 +282,7 @@ def _check_order(sphinx_app, key):
     assert order == [1, 2, 3]
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -273,7 +295,7 @@ def test_example_sorting_default(sphinx_app_wrapper):
     _check_order(sphinx_app, "lines")
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -287,7 +309,7 @@ def test_example_sorting_filesize(sphinx_app_wrapper):
     _check_order(sphinx_app, "filesize")
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -301,7 +323,7 @@ def test_example_sorting_filename(sphinx_app_wrapper):
     _check_order(sphinx_app, "filename")
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -320,12 +342,12 @@ def test_collect_gallery_files(tmpdir, gallery_conf):
     rel_filepaths = [
         "examples/file1.py",
         "examples/test.rst",
-        "examples/README.txt",
+        "examples/GALLERY_HEADER.rst",
         "examples/folder1/file1.py",
         "examples/folder1/file2.py",
         "examples/folder2/file1.py",
-        "tutorials/folder1/subfolder/file1.py",
-        "tutorials/folder2/subfolder/subsubfolder/file1.py",
+        "tutorials/folder1/file1.py",
+        "tutorials/folder2/file1.py",
     ]
 
     abs_paths = [tmpdir.join(rp) for rp in rel_filepaths]
@@ -334,7 +356,9 @@ def test_collect_gallery_files(tmpdir, gallery_conf):
 
     examples_path = tmpdir.join("examples")
     dirs = [examples_path.strpath]
-    collected_files = set(collect_gallery_files(dirs, gallery_conf))
+    collected_files = set(
+        _collect_gallery_files(dirs, gallery_conf, check_filenames=True)
+    )
     expected_files = {
         ap.strpath for ap in abs_paths if re.search(r"examples.*\.py$", ap.strpath)
     }
@@ -343,7 +367,9 @@ def test_collect_gallery_files(tmpdir, gallery_conf):
 
     tutorials_path = tmpdir.join("tutorials")
     dirs = [examples_path.strpath, tutorials_path.strpath]
-    collected_files = set(collect_gallery_files(dirs, gallery_conf))
+    collected_files = set(
+        _collect_gallery_files(dirs, gallery_conf, check_filenames=True)
+    )
     expected_files = {
         ap.strpath for ap in abs_paths if re.search(r".*\.py$", ap.strpath)
     }
@@ -367,17 +393,19 @@ def test_collect_gallery_files_ignore_pattern(tmpdir, gallery_conf):
     gallery_conf["ignore_pattern"] = r"one"
     examples_path = tmpdir.join("examples")
     dirs = [examples_path.strpath]
-    collected_files = set(collect_gallery_files(dirs, gallery_conf))
+    collected_files = set(
+        _collect_gallery_files(dirs, gallery_conf, check_filenames=True)
+    )
     expected_files = {
         ap.strpath
         for ap in abs_paths
-        if re.search(r"one", os.path.basename(ap.strpath)) is None
+        if re.search(r"one", Path(ap.strpath).name) is None
     }
 
     assert collected_files == expected_files
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'backreferences_dir' : os.path.join('modules', 'gen'),
@@ -396,22 +424,20 @@ def test_binder_copy_files(sphinx_app_wrapper):
     sphinx_app = sphinx_app_wrapper.create_sphinx_app()
     gallery_conf = sphinx_app.config.sphinx_gallery_conf
     # Create requirements file
-    with open(os.path.join(sphinx_app.srcdir, "requirements.txt"), "w"):
+    with open(Path(sphinx_app.srcdir, "requirements.txt"), "w"):
         pass
     copy_binder_files(sphinx_app, None)
 
     for i_file in ["plot_1", "plot_2", "plot_3"]:
-        assert os.path.exists(
-            os.path.join(
-                sphinx_app.outdir,
-                "ntbk_folder",
-                gallery_conf["gallery_dirs"][0],
-                i_file + ".ipynb",
-            )
-        )
+        assert Path(
+            sphinx_app.outdir,
+            "ntbk_folder",
+            gallery_conf["gallery_dirs"][0],
+            i_file + ".ipynb",
+        ).exists()
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -419,17 +445,34 @@ sphinx_gallery_conf = {
 }"""
 )
 def test_failing_examples_raise_exception(sphinx_app_wrapper):
-    example_dir = os.path.join(sphinx_app_wrapper.srcdir, "src")
-    with codecs.open(
-        os.path.join(example_dir, "plot_3.py"), "a", encoding="utf-8"
-    ) as fid:
-        fid.write("raise SyntaxError")
+    example_dir = Path(sphinx_app_wrapper.srcdir, "src")
+    bad_line = "print(f'{a[}')"  # never closed bracket inside print -> SyntaxError
+    bad_code = f"""\
+'''
+Failing example
+---------------
+Should emit a syntax error in the second code block.
+'''
+1 + 2
+
+# %%
+# More
+
+{bad_line}
+"""
+    bad_line_no = bad_code.split("\n").index(bad_line) + 1
+    with open(Path(example_dir, "plot_3.py"), "w", encoding="utf-8") as fid:
+        fid.write(bad_code)
     with pytest.raises(ExtensionError) as excinfo:
         sphinx_app_wrapper.build_sphinx_app()
-    assert "Unexpected failing examples" in str(excinfo.value)
+    tb = str(excinfo.value)
+    assert "Unexpected failing examples" in tb
+    # Check traceback points to correct line (see #1301)
+    assert f"line {bad_line_no}" in tb
+    assert bad_line in tb
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -445,7 +488,7 @@ def test_expected_failing_examples_were_executed(sphinx_app_wrapper):
     sphinx_app_wrapper.build_sphinx_app()
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -456,8 +499,8 @@ sphinx_gallery_conf = {
 def test_only_warn_on_example_error(sphinx_app_wrapper):
     """Test behaviour of only_warn_on_example_error flag."""
     example_dir = Path(sphinx_app_wrapper.srcdir) / "src"
-    with codecs.open(example_dir / "plot_3.py", "a", encoding="utf-8") as fid:
-        fid.write("raise ValueError")
+    with open(example_dir / "plot_3.py", "w", encoding="utf-8") as fid:
+        fid.write(f"{MINIMAL_HEADER}raise ValueError")
     sphinx_app = sphinx_app_wrapper.build_sphinx_app()
 
     build_warn = _escape_ansi(sphinx_app._warning.getvalue())
@@ -465,7 +508,7 @@ def test_only_warn_on_example_error(sphinx_app_wrapper):
     assert "WARNING: Here is a summary of the problems" in build_warn
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -477,15 +520,15 @@ def test_only_warn_on_example_error_sphinx_warning(sphinx_app_wrapper):
     """Test behaviour of only_warn_on_example_error flag."""
     sphinx_app_wrapper.kwargs["warningiserror"] = True
     example_dir = Path(sphinx_app_wrapper.srcdir) / "src"
-    with codecs.open(example_dir / "plot_3.py", "a", encoding="utf-8") as fid:
-        fid.write("raise ValueError")
+    with open(example_dir / "plot_3.py", "w", encoding="utf-8") as fid:
+        fid.write(f"{MINIMAL_HEADER}raise ValueError")
     with pytest.raises(SphinxWarning) as excinfo:
         sphinx_app_wrapper.build_sphinx_app()
     exc = _escape_ansi(str(excinfo.value))
     assert "plot_3.py unexpectedly failed to execute" in exc
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'examples_dirs': 'src',
@@ -500,19 +543,20 @@ def test_examples_not_expected_to_pass(sphinx_app_wrapper):
     assert "expected to fail, but not failing" in exc
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 from sphinx_gallery.gen_rst import _sg_call_memory_noop
 
 sphinx_gallery_conf = {
     'show_memory': _sg_call_memory_noop,
+    'examples_dirs': 'src',
     'gallery_dirs': 'ex',
 }"""
 )
 def test_show_memory_callable(sphinx_app_wrapper):
     sphinx_app = sphinx_app_wrapper.build_sphinx_app()
     status = sphinx_app._status.getvalue()
-    assert "0.0 MB" in status
+    assert "0.0 MB" in status, status
 
 
 @pytest.mark.parametrize(
@@ -520,13 +564,13 @@ def test_show_memory_callable(sphinx_app_wrapper):
     [
         pytest.param(
             id="first notebook cell",
-            marks=pytest.mark.conf_file(
+            marks=pytest.mark.add_conf(
                 content="""sphinx_gallery_conf = {'first_notebook_cell': 2,}"""
             ),
         ),
         pytest.param(
             id="last notebook cell",
-            marks=pytest.mark.conf_file(
+            marks=pytest.mark.add_conf(
                 content="""sphinx_gallery_conf = {'last_notebook_cell': 2,}"""
             ),
         ),
@@ -539,7 +583,7 @@ def test_notebook_cell_config(sphinx_app_wrapper):
         fill_gallery_conf_defaults(app, app.config, check_keys=False)
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'backreferences_dir': False,
@@ -547,14 +591,26 @@ sphinx_gallery_conf = {
 )
 def test_backreferences_dir_config(sphinx_app_wrapper):
     """Tests 'backreferences_dir' type checking."""
-    with pytest.raises(
-        ConfigError, match="The 'backreferences_dir' parameter must be of"
-    ):
+    with pytest.raises(ConfigError, match="'backreferences_dir' config allowed types"):
         app = sphinx_app_wrapper.create_sphinx_app()
         fill_gallery_conf_defaults(app, app.config, check_keys=False)
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
+    content="""
+sphinx_gallery_conf = {
+    'examples_dirs': 'src',
+    'gallery_dirs': 'ex',
+}"""
+)
+def test_minigallery_no_backreferences_dir(sphinx_app_wrapper):
+    """Check warning when no backreferences_dir set but minigallery directive used."""
+    sphinx_app = sphinx_app_wrapper.build_sphinx_app()
+    build_warn = sphinx_app._warning.getvalue()
+    assert "'backreferences_dir' config is None, minigallery" in build_warn
+
+
+@pytest.mark.add_conf(
     content="""
 import pathlib
 
@@ -568,6 +624,59 @@ def test_backreferences_dir_pathlib_config(sphinx_app_wrapper):
     fill_gallery_conf_defaults(app, app.config, check_keys=False)
 
 
+@pytest.mark.add_conf(
+    content="""
+sphinx_gallery_conf = {
+    'examples_dirs': 'src',
+    'gallery_dirs': 'ex',
+}"""
+)
+@pytest.mark.add_rst(
+    file="""
+Header
+======
+
+.. minigallery:: index.rst
+"""
+)
+def test_minigallery_not_in_examples_dirs(sphinx_app_wrapper):
+    """Check error when minigallery directive's path input not in `examples_dirs`."""
+    msg = "minigallery directive error: path input 'index.rst'"
+    with pytest.raises(ExtensionError, match=msg):
+        sphinx_app_wrapper.build_sphinx_app()
+
+
+@pytest.mark.add_conf(
+    content="""
+sphinx_gallery_conf = {
+    'examples_dirs': ['src', 'src/sub_folder/sub_sub_folder'],
+    'gallery_dirs': ['ex', 'ex/sub_folder/sub_sub_folder'],
+}"""
+)
+@pytest.mark.add_rst(
+    file="""
+Header
+======
+
+.. minigallery:: src/sub_folder/sub_sub_folder/plot_nested.py
+"""
+)
+def test_minigallery_multi_match(sphinx_app_wrapper):
+    """Check minigallery directive's path input resolution in nested `examples_dirs`.
+
+    When a examples gallery is nested inside another examples gallery, path inputs
+    from the nested gallery should resolve to the nested gallery.
+    """
+    sphinx_app = sphinx_app_wrapper.build_sphinx_app()
+    minigallery_html = Path(sphinx_app.outdir) / "minigallery_test.html"
+    with open(minigallery_html, "r") as fid:
+        mg_html = fid.read()
+    # Check thumbnail correct
+    assert "_images/sphx_glr_plot_nested_thumb.png" in mg_html
+    # Check href correct
+    assert "sphx-glr-ex-sub-folder-sub-sub-folder-plot-nested-py" in mg_html
+
+
 def test_write_computation_times_noop(sphinx_app_wrapper):
     app = sphinx_app_wrapper.create_sphinx_app()
     write_computation_times(app.config.sphinx_gallery_conf, None, [])
@@ -577,7 +686,7 @@ def test_write_api_usage_noop(sphinx_app_wrapper):
     write_api_entry_usage(sphinx_app_wrapper.create_sphinx_app(), list(), None)
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'pypandoc': ['list',],
@@ -586,13 +695,14 @@ sphinx_gallery_conf = {
 def test_pypandoc_config_list(sphinx_app_wrapper):
     """Tests 'pypandoc' type checking."""
     with pytest.raises(
-        ConfigError, match="'pypandoc' parameter must be of type bool or " "dict"
+        ConfigError,
+        match=r"'pypandoc' config allowed types: \['dict', 'bool'\].",
     ):
         app = sphinx_app_wrapper.create_sphinx_app()
         fill_gallery_conf_defaults(app, app.config, check_keys=False)
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'pypandoc': {'bad key': 1},
@@ -607,7 +717,7 @@ def test_pypandoc_config_keys(sphinx_app_wrapper):
         fill_gallery_conf_defaults(app, app.config, check_keys=False)
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 extensions += ['jupyterlite_sphinx']
 
@@ -626,17 +736,15 @@ def test_create_jupyterlite_contents(sphinx_app_wrapper):
     create_jupyterlite_contents(sphinx_app, exception=None)
 
     for i_file in ["plot_1", "plot_2", "plot_3"]:
-        assert os.path.exists(
-            os.path.join(
-                sphinx_app.srcdir,
-                "jupyterlite_contents",
-                gallery_conf["gallery_dirs"][0],
-                i_file + ".ipynb",
-            )
-        )
+        assert Path(
+            sphinx_app.srcdir,
+            "jupyterlite_contents",
+            gallery_conf["gallery_dirs"][0],
+            i_file + ".ipynb",
+        ).exists()
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 extensions += ['jupyterlite_sphinx']
 
@@ -656,17 +764,15 @@ def test_create_jupyterlite_contents_non_default_contents(sphinx_app_wrapper):
     create_jupyterlite_contents(sphinx_app, exception=None)
 
     for i_file in ["plot_1", "plot_2", "plot_3"]:
-        assert os.path.exists(
-            os.path.join(
-                sphinx_app.srcdir,
-                "this_is_the_contents_dir",
-                gallery_conf["gallery_dirs"][0],
-                i_file + ".ipynb",
-            )
-        )
+        assert Path(
+            sphinx_app.srcdir,
+            "this_is_the_contents_dir",
+            gallery_conf["gallery_dirs"][0],
+            i_file + ".ipynb",
+        ).exists()
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 sphinx_gallery_conf = {
     'backreferences_dir' : os.path.join('modules', 'gen'),
@@ -682,10 +788,10 @@ def test_create_jupyterlite_contents_without_jupyterlite_sphinx_loaded(
     sphinx_app = sphinx_app_wrapper.create_sphinx_app()
 
     create_jupyterlite_contents(sphinx_app, exception=None)
-    assert not os.path.exists(os.path.join(sphinx_app.srcdir, "jupyterlite_contents"))
+    assert not Path(sphinx_app.srcdir, "jupyterlite_contents").exists()
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 extensions += ['jupyterlite_sphinx']
 
@@ -707,10 +813,10 @@ def test_create_jupyterlite_contents_with_jupyterlite_disabled_via_config(
     sphinx_app = sphinx_app_wrapper.create_sphinx_app()
 
     create_jupyterlite_contents(sphinx_app, exception=None)
-    assert not os.path.exists(os.path.join(sphinx_app.outdir, "jupyterlite_contents"))
+    assert not Path(sphinx_app.outdir, "jupyterlite_contents").exists()
 
 
-@pytest.mark.conf_file(
+@pytest.mark.add_conf(
     content="""
 extensions += ['jupyterlite_sphinx']
 
@@ -741,13 +847,13 @@ def test_create_jupyterlite_contents_with_modification(sphinx_app_wrapper):
     create_jupyterlite_contents(sphinx_app, exception=None)
 
     for i_file in ["plot_1", "plot_2", "plot_3"]:
-        notebook_filename = os.path.join(
+        notebook_filename = Path(
             sphinx_app.srcdir,
             "jupyterlite_contents",
             gallery_conf["gallery_dirs"][0],
             i_file + ".ipynb",
         )
-        assert os.path.exists(notebook_filename)
+        assert notebook_filename.exists()
 
         with open(notebook_filename) as f:
             notebook_content = json.load(f)
