@@ -4,30 +4,35 @@
 
 import ast
 import codecs
+import codeop
 import importlib
 import io
 import logging
-import tempfile
-import re
 import os
+import re
 import shutil
-import sys
-from unittest import mock
+import tempfile
 import zipfile
-import codeop
+from pathlib import Path
+from unittest import mock
 
 import pytest
-
 from sphinx.errors import ExtensionError
+
 import sphinx_gallery.gen_rst as sg
 from sphinx_gallery import downloads
 from sphinx_gallery.gen_gallery import (
-    generate_dir_rst,
     _update_gallery_conf_exclude_implicit_doc,
+    generate_dir_rst,
 )
-from sphinx_gallery.scrapers import ImagePathIterator, figure_rst
 from sphinx_gallery.interactive_example import check_binder_conf
 
+# TODO: The tests of this method should probably be moved to test_py_source_parser.py
+from sphinx_gallery.py_source_parser import Block, split_code_and_text_blocks
+from sphinx_gallery.scrapers import ImagePathIterator, figure_rst
+
+root = Path(__file__).parents[2]
+
 CONTENT = [
     '"""',
     "================",
@@ -65,7 +70,9 @@ CONTENT = [
 
 def test_split_code_and_text_blocks():
     """Test if a known example file gets properly split."""
-    file_conf, blocks = sg.split_code_and_text_blocks("examples/no_output/just_code.py")
+    file_conf, blocks = split_code_and_text_blocks(
+        root / "examples" / "no_output" / "just_code.py",
+    )
 
     assert file_conf == {}
     assert blocks[0][0] == "text"
@@ -77,12 +84,13 @@ def test_bug_cases_of_notebook_syntax():
 
     `plot_parse.py` uses both '#'s' and '#%%' as cell separators.
     """
-    with open("sphinx_gallery/tests/reference_parse.txt") as reference:
-        ref_blocks = ast.literal_eval(reference.read())
-        file_conf, blocks = sg.split_code_and_text_blocks("tutorials/plot_parse.py")
+    ref_blocks = ast.literal_eval(
+        (Path(__file__).parent / "reference_parse.txt").read_text("utf-8")
+    )
+    file_conf, blocks = split_code_and_text_blocks(root / "tutorials" / "plot_parse.py")
 
-        assert file_conf == {}
-        assert blocks == ref_blocks
+    assert file_conf == {}
+    assert blocks == ref_blocks
 
 
 def test_direct_comment_after_docstring():
@@ -101,7 +109,7 @@ def test_direct_comment_after_docstring():
             )
         )
     try:
-        file_conf, result = sg.split_code_and_text_blocks(f.name)
+        file_conf, result = split_code_and_text_blocks(f.name)
     finally:
         os.remove(f.name)
 
@@ -139,7 +147,7 @@ def test_final_rst_last_word(tmpdir):
             )
         )
 
-    file_conf, result = sg.split_code_and_text_blocks(f.name)
+    file_conf, result = split_code_and_text_blocks(f.name)
 
     assert file_conf == {}
     expected_result = [
@@ -171,7 +179,7 @@ def test_rst_block_after_docstring(gallery_conf, tmpdir):
                 ]
             )
         )
-    file_conf, blocks = sg.split_code_and_text_blocks(filename)
+    file_conf, blocks = split_code_and_text_blocks(filename)
 
     assert file_conf == {}
     assert len(blocks) == 4
@@ -224,7 +232,7 @@ def test_rst_empty_code_block(gallery_conf, tmpdir):
                 ]
             )
         )
-    file_conf, blocks = sg.split_code_and_text_blocks(filename)
+    file_conf, blocks = split_code_and_text_blocks(filename)
 
     assert file_conf == {}
     assert len(blocks) == 3
@@ -278,7 +286,7 @@ a = 1.
 b = 'foo'
 """
         )
-    file_conf, blocks = sg.split_code_and_text_blocks(filename)
+    file_conf, blocks = split_code_and_text_blocks(filename)
     assert len(blocks) == 2
     assert blocks[0][0] == "text"
     assert blocks[1][0] == "code"
@@ -311,9 +319,7 @@ def test_extract_intro_and_title():
     intro, title = sg.extract_intro_and_title("<string>", "\n".join(CONTENT[1:10]))
     assert title == "Docstring header"
     assert "Docstring" not in intro
-    assert (
-        intro == "This is the description of the example which goes on and on, Óscar"
-    )  # noqa
+    assert intro == "This is the description of the example which goes on and on, Óscar"  # noqa
     assert "second paragraph" not in intro
 
     # SG incorrectly grabbing description when a label is defined (gh-232)
@@ -346,15 +352,14 @@ def test_extract_intro_and_title():
     )  # noqa: E501
     assert title == '"-`Header"-with:; punct mark\'s'
 
-    # Long intro paragraph gets shortened
+    # Long intro paragraph are preserved
     intro_paragraph = "\n".join(["this is one line" for _ in range(10)])
     intro, _ = sg.extract_intro_and_title(
         "filename", "Title\n-----\n\n" + intro_paragraph
     )
     assert len(intro_paragraph) > 100
-    assert len(intro) < 100
-    assert intro.endswith("...")
-    assert intro_paragraph.replace("\n", " ")[:95] == intro[:95]
+    assert len(intro) > 100
+    assert intro_paragraph.replace("\n", " ") == intro
 
     # Errors
     with pytest.raises(ExtensionError, match="should have a header"):
@@ -370,26 +375,20 @@ def test_extract_intro_and_title():
         ["t", "ea8a570e9f3afc0a7c3f2a17a48b8047"],
     ),
 )
-def test_md5sums(mode, expected_md5):
+def test_md5sums(mode, expected_md5, tmp_path):
     """Test md5sum check functions work on know file content."""
     file_content = b"Local test\r\n"
-    with tempfile.NamedTemporaryFile("wb", delete=False) as f:
-        f.write(file_content)
-    try:
-        file_md5 = sg.get_md5sum(f.name, mode)
-        # verify correct md5sum
-        assert file_md5 == expected_md5
-        # False because is a new file
-        assert not sg.md5sum_is_current(f.name)
-        # Write md5sum to file to check is current
-        with open(f.name + ".md5", "w") as file_checksum:
-            file_checksum.write(file_md5)
-        try:
-            assert sg.md5sum_is_current(f.name, mode)
-        finally:
-            os.remove(f.name + ".md5")
-    finally:
-        os.remove(f.name)
+    fname = tmp_path / "test"
+    fname.write_bytes(file_content)
+    file_md5 = sg.get_md5sum(fname, mode)
+    # verify correct md5sum
+    assert file_md5 == expected_md5, mode
+    # False because is a new file
+    assert not sg.md5sum_is_current(fname), mode
+    # Write md5sum to file to check is current
+    with open(str(fname) + ".md5", "w") as file_checksum:
+        file_checksum.write(file_md5)
+    assert sg.md5sum_is_current(fname, mode), mode
 
 
 @pytest.mark.parametrize(
@@ -471,9 +470,18 @@ def _generate_rst(gallery_conf, fname, content):
         os.path.join(gallery_conf["examples_dir"], fname), mode="w", encoding="utf-8"
     ) as f:
         f.write("\n".join(content))
+    with codecs.open(
+        os.path.join(gallery_conf["examples_dir"], "README.txt"), "w", "utf8"
+    ):
+        pass
+
     # generate rst file
-    sg.generate_file_rst(
-        fname, gallery_conf["gallery_dir"], gallery_conf["examples_dir"], gallery_conf
+    generate_dir_rst(
+        gallery_conf["examples_dir"],
+        gallery_conf["gallery_dir"],
+        gallery_conf,
+        set(),
+        is_subsection=False,
     )
     # read rst file and check if it contains code output
     rst_fname = os.path.splitext(fname)[0] + ".rst"
@@ -493,9 +501,7 @@ Plot.
 """
 import matplotlib.pyplot as plt
 plt.plot([0, 1], [0, 1])
-'''.split(
-    "\n"
-)
+'''.split("\n")
 
 
 def _alpha_mpl_scraper(block, block_vars, gallery_conf):
@@ -507,7 +513,7 @@ def _alpha_mpl_scraper(block, block_vars, gallery_conf):
         fig = plt.figure(fig_num)
         assert image_path.endswith(".png")
         # use format that does not support alpha
-        image_path = image_path[:-3] + "jpg"
+        image_path = Path(image_path).with_suffix(".jpg")
         fig.savefig(image_path)
         image_paths.append(image_path)
     plt.close("all")
@@ -561,6 +567,7 @@ def test_final_empty_block(gallery_conf, req_pil):
     """
     content_block = CONTENT + ["# %%", "", "# sphinx_gallery_line_numbers = True"]
     gallery_conf["remove_config_comments"] = True
+    gallery_conf["min_reported_time"] = -1  # Force timing info to be shown
     rst = _generate_rst(gallery_conf, "test.py", content_block)
     want = "RuntimeWarning)\n\n\n.. rst-class:: sphx-glr-timing"
     assert want in rst
@@ -589,9 +596,7 @@ EXCLUDE_CONTENT = '''
 import numpy
 numpy.pi
 numpy.e
-'''.split(
-    "\n"
-)
+'''.split("\n")
 
 
 @pytest.mark.parametrize(
@@ -617,29 +622,41 @@ def test_exclude_implicit(gallery_conf, exclusion, expected, monkeypatch, req_pi
         gallery_conf["exclude_implicit_doc"] = exclusion
         _update_gallery_conf_exclude_implicit_doc(gallery_conf)
     _generate_rst(gallery_conf, "test_exclude_implicit.py", EXCLUDE_CONTENT)
-    if sys.version_info >= (3, 8, 0):
-        assert mock_write_backreferences.call_args.args[0] == expected
-    else:
-        assert mock_write_backreferences.call_args[0][0] == expected
+    assert mock_write_backreferences.call_args.args[0] == expected
 
 
-@pytest.mark.parametrize("ext", (".txt", ".rst", ".bad"))
-def test_gen_dir_rst(gallery_conf, ext):
+@pytest.mark.parametrize(
+    "gallery_header",
+    [
+        "GALLERY_HEADER.rst",
+        "README.txt",
+        "README.rst",
+    ],
+)
+def test_gen_dir_rst(gallery_conf, gallery_header):
     """Test gen_dir_rst."""
-    print(os.listdir(gallery_conf["examples_dir"]))
-    fname_readme = os.path.join(gallery_conf["src_dir"], "README.txt")
-    with open(fname_readme, "wb") as fid:
+    with open(os.path.join(gallery_conf["src_dir"], gallery_header), "wb") as fid:
         fid.write("Testing\n=======\n\nÓscar here.".encode())
-    fname_out = os.path.splitext(fname_readme)[0] + ext
-    if fname_readme != fname_out:
-        shutil.move(fname_readme, fname_out)
-    args = (gallery_conf["src_dir"], gallery_conf["gallery_dir"], gallery_conf, [])
-    if ext == ".bad":  # not found with correct ext
-        with pytest.raises(ExtensionError, match="does not have a README"):
-            generate_dir_rst(*args)
-    else:
-        out = generate_dir_rst(*args)
-        assert "Óscar here" in out[1]
+    out = generate_dir_rst(
+        gallery_conf["src_dir"], gallery_conf["gallery_dir"], gallery_conf, []
+    )
+    assert "Óscar here" in out[1]
+
+
+@pytest.mark.parametrize(
+    "gallery_header",
+    [
+        "GALLERY_HEADER.bad",
+        "README.bad",
+    ],
+)
+def test_gen_dir_rst_invalid_header(gallery_conf, gallery_header):
+    with open(os.path.join(gallery_conf["src_dir"], gallery_header), "wb") as fid:
+        fid.write("Testing\n=======\n\nÓscar here.".encode())
+    with pytest.raises(ExtensionError, match="does not have a GALLERY_HEADER"):
+        generate_dir_rst(
+            gallery_conf["src_dir"], gallery_conf["gallery_dir"], gallery_conf, []
+        )
 
 
 def test_pattern_matching(gallery_conf, log_collector, req_pil):
@@ -685,7 +702,7 @@ def test_thumbnail_number(test_str):
     with tempfile.NamedTemporaryFile("w", delete=False) as f:
         f.write("\n".join(['"Docstring"', test_str]))
     try:
-        file_conf, blocks = sg.split_code_and_text_blocks(f.name)
+        file_conf, blocks = split_code_and_text_blocks(f.name)
     finally:
         os.remove(f.name)
     assert file_conf == {"thumbnail_number": 2}
@@ -705,14 +722,14 @@ def test_thumbnail_path(test_str):
     with tempfile.NamedTemporaryFile("w", delete=False) as f:
         f.write("\n".join(['"Docstring"', test_str]))
     try:
-        file_conf, blocks = sg.split_code_and_text_blocks(f.name)
+        file_conf, blocks = split_code_and_text_blocks(f.name)
     finally:
         os.remove(f.name)
     assert file_conf == {"thumbnail_path": "_static/demo.png"}
 
 
-def test_zip_notebooks(gallery_conf):
-    """Test generated zipfiles are not corrupt."""
+def test_zip_python(gallery_conf):
+    """Test generated zipfiles are not corrupt and have expected name and contents."""
     gallery_conf.update(examples_dir=os.path.join(gallery_conf["src_dir"], "examples"))
     shutil.copytree(
         os.path.join(os.path.dirname(__file__), "tinybuild", "examples"),
@@ -720,9 +737,32 @@ def test_zip_notebooks(gallery_conf):
     )
     examples = downloads.list_downloadable_sources(gallery_conf["examples_dir"])
     zipfilepath = downloads.python_zip(examples, gallery_conf["gallery_dir"])
+    assert zipfilepath.endswith("_python.zip")
     zipf = zipfile.ZipFile(zipfilepath)
     check = zipf.testzip()
     assert not check, f"Bad file in zipfile: {check}"
+    filenames = {item.filename for item in zipf.filelist}
+    assert "examples/plot_command_line_args.py" in filenames
+    assert "examples/julia_sample.jl" not in filenames
+
+
+def test_zip_mixed_source(gallery_conf):
+    """Test generated zipfiles are not corrupt and have expected name and contents."""
+    gallery_conf.update(examples_dir=os.path.join(gallery_conf["src_dir"], "examples"))
+    shutil.copytree(
+        os.path.join(os.path.dirname(__file__), "tinybuild", "examples"),
+        gallery_conf["examples_dir"],
+    )
+    examples = downloads.list_downloadable_sources(
+        gallery_conf["examples_dir"], (".py", ".jl")
+    )
+    zipfilepath = downloads.python_zip(examples, gallery_conf["gallery_dir"], None)
+    zipf = zipfile.ZipFile(zipfilepath)
+    check = zipf.testzip()
+    assert not check, f"Bad file in zipfile: {check}"
+    filenames = {item.filename for item in zipf.filelist}
+    assert "examples/plot_command_line_args.py" in filenames
+    assert "examples/julia_sample.jl" in filenames
 
 
 def test_rst_example(gallery_conf):
@@ -738,6 +778,7 @@ def test_rst_example(gallery_conf):
         }
     )
     gallery_conf.update(binder=binder_conf)
+    gallery_conf["min_reported_time"] = -1
 
     example_file = os.path.join(gallery_conf["gallery_dir"], "plot.py")
     sg.save_rst_example("example_rst", example_file, 0, 0, gallery_conf)
@@ -776,7 +817,7 @@ def test_output_indentation(gallery_conf, script_vars):
 
     test_string = r"\n".join(["  A B", "A 1 2", "B 3 4"])
     code = "print('" + test_string + "')"
-    code_block = ("code", code, 1)
+    code_block = Block("code", code, 1)
     file_conf = {}
     output = sg.execute_code_block(
         compiler, code_block, None, script_vars, gallery_conf, file_conf
@@ -796,7 +837,7 @@ def test_output_no_ansi(gallery_conf, script_vars):
     compiler = codeop.Compile()
 
     code = 'print("\033[94m0.25")'
-    code_block = ("code", code, 1)
+    code_block = Block("code", code, 1)
     file_conf = {}
     output = sg.execute_code_block(
         compiler, code_block, None, script_vars, gallery_conf, file_conf
@@ -819,7 +860,7 @@ def test_absl_logging(gallery_conf, script_vars):
     compiler = codeop.Compile()
     code = 'from absl import logging\nlogging.info("Should not crash!")'
     sg.execute_code_block(
-        compiler, ("code", code, 1), None, script_vars, gallery_conf, {}
+        compiler, Block("code", code, 1), None, script_vars, gallery_conf, {}
     )
 
     # Previously, absl crashed during shutdown, after all tests had run, so
@@ -833,7 +874,7 @@ def test_empty_output_box(gallery_conf, script_vars):
     gallery_conf.update(image_scrapers=())
     compiler = codeop.Compile()
 
-    code_block = ("code", "print(__doc__)", 1)
+    code_block = Block("code", "print(__doc__)", 1)
     file_conf = {}
 
     output = sg.execute_code_block(
@@ -991,7 +1032,7 @@ def test_capture_repr(
 ):
     """Tests output capturing with various capture_repr settings."""
     compiler = codeop.Compile()
-    code_block = ("code", code, 1)
+    code_block = Block("code", code, 1)
     gallery_conf["capture_repr"] = capture_repr
     file_conf = {}
     output = sg.execute_code_block(
@@ -1018,7 +1059,7 @@ def test_per_file_capture_repr(
 ):
     """Tests that per file capture_repr overrides gallery_conf."""
     compiler = codeop.Compile()
-    code_block = ("code", "a=2\n2", 1)
+    code_block = Block("code", "a=2\n2", 1)
     gallery_conf["capture_repr"] = caprepr_gallery
     file_conf = {"capture_repr": caprepr_file}
     output = sg.execute_code_block(
@@ -1030,7 +1071,7 @@ def test_per_file_capture_repr(
 def test_ignore_repr_types(gallery_conf, req_mpl, req_pil, script_vars):
     """Tests output capturing with various capture_repr settings."""
     compiler = codeop.Compile()
-    code_block = ("code", "a=2\na", 1)
+    code_block = Block("code", "a=2\na", 1)
     gallery_conf["ignore_repr_types"] = r"int"
     file_conf = {}
     output = sg.execute_code_block(
@@ -1168,7 +1209,7 @@ def test_textio_compat(log_collector_wrap):
     assert tee.writable()
 
 
-def test_fileno(monkeypatch, log_collector_wrap):
+def test_fileno(log_collector_wrap):
     _, _, tee, _ = log_collector_wrap
     with pytest.raises(io.UnsupportedOperation):
         tee.fileno()
@@ -1198,6 +1239,5 @@ def test_newlines(log_collector_wrap):
     assert tee.newlines == tee.output.newlines
 
 
-# TODO: test that broken thumbnail does appear when needed
 # TODO: test that examples are executed after a no-plot and produce
 #       the correct image in the thumbnail
