@@ -9,5 +9,5 @@ A test with a really long title to make the filesize larger
 :filename=3:title=1:lines=2:filesize=3:
 """
 
-print('foo')
-print('bar')
+print("foo")
+print("bar")
