@@ -9,10 +9,11 @@ test this... we require Python 3 nowadays so the __future__ statements there
 don't do anything. So for now let's repurpose this to look at some
 backreferences. We should probably also change the filename in another PR!
 """
+
 # sphinx_gallery_thumbnail_path = '_static_nonstandard/demo.png'
 
+from sphinx_gallery.scrapers import clean_modules, figure_rst
 from sphinx_gallery.sorting import ExplicitOrder
-from sphinx_gallery.scrapers import figure_rst, clean_modules
 
 ExplicitOrder([])  # must actually be used to become a backref target!
 
