@@ -2,17 +2,22 @@
 
 Miscellaneous utilities.
 """
+
 # Author: Eric Larson
 # License: 3-clause BSD
 
-
 import hashlib
+import json
 import os
-from shutil import move, copyfile
+import re
 import subprocess
+import zipfile
+from functools import partial
+from pathlib import Path
+from shutil import copyfile, move
 
-from sphinx.errors import ExtensionError
 import sphinx.util
+from sphinx.errors import ExtensionError
 
 try:
     from sphinx.util.display import status_iterator  # noqa: F401
@@ -23,6 +28,10 @@ except Exception:  # Sphinx < 6
 logger = sphinx.util.logging.getLogger("sphinx-gallery")
 
 
+# Text writing kwargs for builtins.open
+_W_KW = dict(encoding="utf-8", newline="\n")
+
+
 def _get_image():
     try:
         from PIL import Image
@@ -97,6 +106,7 @@ def optipng(fname, args=()):
     args : tuple
         Extra command-line arguments, such as ``['-o7']``.
     """
+    fname = str(fname)
     if fname.endswith(".png"):
         # -o7 because this is what CPython used
         # https://github.com/python/cpython/pull/8032
@@ -121,19 +131,6 @@ def _has_optipng():
         return True
 
 
-def replace_py_ipynb(fname):
-    """Replace '.py' extension in filename with '.ipynb'."""
-    fname_prefix, extension = os.path.splitext(fname)
-    allowed_extension = ".py"
-    if extension != allowed_extension:
-        raise ValueError(
-            f"Unrecognized file extension, expected {allowed_extension}, "
-            f"got {extension}"
-        )
-    new_extension = ".ipynb"
-    return f"{fname_prefix}{new_extension}"
-
-
 def get_md5sum(src_file, mode="b"):
     """Returns md5sum of file.
 
@@ -149,21 +146,35 @@ def get_md5sum(src_file, mode="b"):
         kwargs = {"errors": "surrogateescape", "encoding": "utf-8"}
     else:
         kwargs = {}
-    with open(src_file, "r" + mode, **kwargs) as src_data:
+    # Universal newline mode is intentional here
+    with open(src_file, f"r{mode}", **kwargs) as src_data:
         src_content = src_data.read()
         if mode == "t":
             src_content = src_content.encode(**kwargs)
         return hashlib.md5(src_content).hexdigest()
 
 
-def _replace_md5(fname_new, fname_old=None, method="move", mode="b"):
+def _replace_md5(fname_new, fname_old=None, *, method="move", mode="b", check="md5"):
+    fname_new = str(fname_new)  # convert possible Path
     assert method in ("move", "copy")
     if fname_old is None:
         assert fname_new.endswith(".new")
         fname_old = os.path.splitext(fname_new)[0]
     replace = True
     if os.path.isfile(fname_old):
-        if get_md5sum(fname_old, mode) == get_md5sum(fname_new, mode):
+        if check == "md5":  # default
+            func = partial(get_md5sum, mode=mode)
+        else:
+            assert check == "json"
+
+            def func(x):
+                return json.loads(Path(x).read_text("utf-8"))
+
+        try:
+            equiv = func(fname_old) == func(fname_new)
+        except Exception:  # e.g., old JSON file is a problem
+            equiv = False
+        if equiv:
             replace = False
             if method == "move":
                 os.remove(fname_new)
@@ -177,6 +188,88 @@ def _replace_md5(fname_new, fname_old=None, method="move", mode="b"):
     assert os.path.isfile(fname_old)
 
 
+def check_duplicate_filenames(files):
+    """Check for duplicate filenames across gallery directories."""
+    # Check whether we'll have duplicates
+    used_names = set()
+    dup_names = list()
+
+    for this_file in files:
+        this_fname = os.path.basename(this_file)
+        if this_fname in used_names:
+            dup_names.append(this_file)
+        else:
+            used_names.add(this_fname)
+
+    if len(dup_names) > 0:
+        logger.warning(
+            "Duplicate example file name(s) found. Having duplicate file "
+            "names will break some links. "
+            "List of files: %s",
+            sorted(dup_names),
+        )
+
+
+def check_spaces_in_filenames(files):
+    """Check for spaces in filenames across example directories."""
+    regex = re.compile(r"[\s]")
+    files_with_space = list(filter(regex.search, files))
+    if files_with_space:
+        logger.warning(
+            "Example file name(s) with spaces found. Having spaces in "
+            "file names will break some links. "
+            "List of files: %s",
+            sorted(files_with_space),
+        )
+
+
+def _collect_gallery_files(examples_dirs, gallery_conf, check_filenames=False):
+    """Collect files with `example_extensions`, accounting for `ignore_pattern`.
+
+    If `check_filenames` we check one level of sub-folders as well as root
+    `example_dirs` for gallery example files. We then check for duplicate and
+    spaces in full file paths.
+    """
+    exts = gallery_conf["example_extensions"]
+    max_depth = 1 if check_filenames else 0
+    files = []
+    for example_dir in examples_dirs:
+        example_depth = os.path.abspath(example_dir).count(os.sep)
+        for root, _, filenames in os.walk(example_dir):
+            root = os.path.normpath(root)
+            if (root.count(os.sep) - example_depth) > max_depth:
+                break
+            for filename in filenames:
+                if (s := Path(filename).suffix) and s in exts:
+                    if re.search(gallery_conf["ignore_pattern"], filename) is None:
+                        file = filename
+                        if check_filenames:
+                            file = os.path.join(root, filename)
+                        files.append(file)
+    if check_filenames:
+        check_duplicate_filenames(files)
+        check_spaces_in_filenames(files)
+    return files
+
+
+def zip_files(file_list, zipname, relative_to, extension=None):
+    """
+    Creates a zip file with the given files.
+
+    A zip file named `zipname` will be created containing the files listed in
+    `file_list`. The zip file contents will be stored with their paths stripped to be
+    relative to `relative_to`.
+    """
+    zipname_new = str(zipname) + ".new"
+    with zipfile.ZipFile(zipname_new, mode="w") as zipf:
+        for fname in file_list:
+            if extension is not None:
+                fname = os.path.splitext(fname)[0] + extension
+            zipf.write(fname, os.path.relpath(fname, relative_to))
+    _replace_md5(zipname_new)
+    return zipname
+
+
 def _has_pypandoc():
     """Check if pypandoc package available."""
     try:
@@ -200,3 +293,26 @@ def _has_graphviz():
         )
         return False
     return True
+
+
+def _escape_ansi(s):
+    """Remove ANSI terminal formatting characters from a string."""
+    return re.sub(r"(?:\x1B[@-_]|[\x80-\x9F])[0-?]*[ -/]*[@-~]", "", s)
+
+
+def _format_toctree(items, includehidden=False):
+    """Format a toc tree."""
+    st = """
+.. toctree::
+   :hidden:"""
+    if includehidden:
+        st += """
+   :includehidden:
+"""
+    st += """
+
+   {}\n""".format("\n   ".join(items))
+
+    st += "\n"
+
+    return st
